#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__ = [] #utilisé pour n'autoriser l'accès à aucune fonctions

def GetAlphabet(AlphaList, OccList, threshold):
#demande la liste des caractères trouvés sur la pages et les occurences de ces mêmes caractères
    alphabet = []
    base = ''
    counter = 0
    #initialise les varables utiles

    for i in AlphaList:
        if len(OccList[counter]) >= threshold: #si le caractère apparait plus de n fois sur la page web
            alphabet.append(i)          #on ajoute le caractère dans le dictionnaire de caractères
        counter +=1
    return alphabet
    # retourne un nouvel alphabet contenant tout les caractères ayant au moins n occurences avec n=threshold

def HexToBase(Hexa, AlphabetList, OccurList, threshold=10):
    number = int(Hexa,16) #base 16 vers base 10
    alphabet = GetAlphabet(AlphabetList, OccurList, threshold)
    base = ''

#génère une base alphanumérique afin de réduire la taille des fichier et de réduire les répétitions d'occurences
    while number != 0:
        number,i = divmod(number,len(alphabet)) # divise le nombre par la base et met le reste dans i
        base = alphabet[i] + base               # ajoute a la base le caractère correspondant au code i
    return base

def BaseToHex(base, AlphabetList, OccurList, threshold=10): #inverse de hex to base
    alphabet = GetAlphabet(AlphabetList, OccurList, threshold)
    out = 0
    for i in base: # on multiplie la sortie par la base en ajoutant la valeur correspondant au caractère
        out = out * len(alphabet) + alphabet.index(i)
    return str(hex(out))[2:] #retourne tout sauf les deux premiers caractères "0x"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import binascii         #https://docs.python.org/3/library/binascii.html
import hashlib          #https://docs.python.org/fr/3.6/library/hashlib.html
from random import randint as rnd
from NimingCypher.BaseConverter import HexToBase, BaseToHex
import lzma             #utilitaire de compression de données basé sur l'algorithme LZMA

__all__ = []    #utilisé pour n'autoriser l'accès à aucune fonctions
threshold = 10  #utilisé pour définir le nb d'occurrence minimale nécessaire pour l'utilisation dans la base

def chiffrer(Clear_tx,listes_globales):
    liste_char, liste_pos = listes_globales
    final_tx = ""
    pas_trouver = 0
    for ch in Clear_tx: #pour chaque caractères
        if ch in liste_char: #si le carctère est dans la liste des caractères
            idx = liste_char.index(ch)  #on trouve son index dans la liste
            #on récupère la liste de toute les occurences de ce caractère
            #qui possède le même index que la première liste
            lpos = liste_pos[idx]

            #on choisit une de ces occurence aléatoirement
            position = int(lpos[(rnd(0,len(lpos)-1))])

            #on converti la position en hexa et on l'ajoute
            final_tx += str(hex(position))[2:] + " "
        else: #si le caractère n'existe pas dans la liste
            pas_trouver += 1
            final_tx += "0000 "
    print(str(pas_trouver) + ' caractères non trouvés !')
    return(final_tx[:-1]) #retourne tout sauf le dernier caractère " "

def dechiffrer(Encrypted_tx,TextFromWeb):
    OutText = ""
    for ch in Encrypted_tx.split(" "): #pour chaque bloc hexa
        if ch != '': #vérifie que le bloc soit bien existant
            try:
                OutText+=str(TextFromWeb[int(ch,16)]) #retourne le caractère qui correspond à la position en hexa
            except:
                print("Erreur lors du déchiffrement ")
                OutText+= '?'
    return OutText

def GenSum(FileBytes): #génère un hash en sha256 pour la vérification des fichiers appelé sum
    return hashlib.sha256(FileBytes).hexdigest()

def FileToHex(chemin, alphabet, occurlist):
    file = open(chemin,"rb") # r --> read ; b --> byte
    fop = file.read() #stocke le fichier en bytes dans fop
    file.close()
    hx = binascii.hexlify(fop) #on transforme les bytes en hexa
    base = HexToBase(hx, alphabet, occurlist, threshold) #on utilise la base personnalisée pour économiser de la place
    return base

def HexToFile(BaseFile, path, alphabet, occurlist):#inverse de FileToHex
    file = open(path,"wb+") # w --> write ; b --> byte ; + --> créer le fichier si inexistant
    Hx = BaseToHex(BaseFile, alphabet, occurlist, threshold) #base perso vers hexa
    file.write(binascii.unhexlify(Hx))#on écrit le fichier en bytes converti auparavant
    file.close()
    return "Hex to file OK !"

def ChiffrFile(location, listes): #chiffre le fichier
    liste_char,liste_pos = listes
    hxTxt = FileToHex(location, liste_char, liste_pos)
    return chiffrer(hxTxt, listes)

def DeChiffrFile(textencrypted, textfromweb, listes, path): #dechiffre le fichier
    liste_char,liste_pos = listes
    dechTx = dechiffrer(textencrypted,textfromweb)
    return(HexToFile(dechTx, path, liste_char, liste_pos))

def Compress(data):
    dte = lzma.compress(data)
    return dte

def Decompress(data):
    dtee = lzma.decompress(data)
    return dtee



#!/usr/bin/env python

##    Usage:
##
##    >>> from NimingCypher import NCrypter
##    >>> crypter = NCrypter("https://key.com")
##    >>> encrypted_str = crypter.crypt_text("simple string")
##    >>> print(encrypted_str)

from NimingCypher.Chiffrement import ChiffrFile, DeChiffrFile, Compress, Decompress, GenSum, dechiffrer, chiffrer
from NimingCypher.GetWebText import getsite, tri_char

#Toutes les explications sont dans les fichiers correspondants
#==================================================================================================================================

class NCrypter:

    def __init__(self, clef):
        result = self.setkey(clef)
        if result == False:
            return False

    def setkey(self, clef):
        self._key = clef #_key pour rendre le paramètre 'key' privé
        try:
            self.textfromweb = getsite(self._key)
            self.charlist = tri_char(self.textfromweb)
            #Initialisation terminée !
        except:
            #Erreur de connexion !
            return False

    def crypt_file(self, path):
        file = open(path,"rb")
        filebytes = file.read()
        file.close()
        Sum = GenSum(filebytes)
        del filebytes
        fl = Compress(ChiffrFile(path,self.charlist))
        return (Sum, fl) #hash sha256 et le fichier chiffré

    def decrypt_file(self, textencrypte, path, open_end=False):
        from os import startfile
        decompress_data = Decompress(textencrypte)
        DeChiffrFile(decompress_data, self.textfromweb, self.charlist, path)
        if open_end == True:
            startfile(path)     #ouvre le fichier
        file = open(path,"rb")    #pour le sum
        filebytes = file.read()
        Sum = GenSum(filebytes)
        file.close()
        del filebytes
        return Sum

    def crypt_text(self, text):
        return chiffrer(text,self.charlist)

    def decrypt_text(self, text):
        return dechiffrer(text, self.textfromweb)

#==================================================================================================================================
